unit IAcolorRichEdit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls,IAColorRichItem,IAColorTimer;
const
TIASpaseSet = [chr(0)..chr(47),chr(58)..chr(64),chr(91)..chr(94),chr(123)..chr(125)];

type
TIAdiapasonColor = record
  Dbegin : integer;
  Dend   : integer;
  Dindex : integer;
end;

  TIAcolorRichEdit = class(TRichEdit)
  private
    FColorProperties : TIAColorRichItems;
    FonChange: TNotifyEvent;
    CurrPos,Currl : integer;
    CurrDelta : integer;
    FMaxUndoS : integer;
    FUnderColor : boolean;
    Fstring,FStringU : string;
    FColorTimer : TIAcolortimer;

    FS  : array of integer;
    FSc : array of integer;

    FDifference : array of TIAdiapasonColor;
    FColorRange: Integer;
    FColorDelay: Word;
    FColorRangePrimary: Integer;
    FLastPosition : Integer;

    procedure InternalGhanged(Sender: TObject);
    procedure FontGhanged(Sender: TObject);
    procedure TryColorItems(DoAtPos : Boolean = false);
    procedure ColorNumber(Index : integer);
    procedure ColorNumberWord(Index : integer);
    procedure ColorNumberRange(Index : integer);
    procedure ColorNumberBrackets(Index : integer);

    procedure ColorAndSelect(Adiff :  TIAdiapasonColor);
    function  FindText1(StartPos, Length1: Integer; Options: TSearchTypes;SS: string): Integer;register;
    procedure RedefineColor(Index,L,Connect : integer);
    procedure AddDifference(Adiff : TIAdiapasonColor);
    procedure CMCOLORIT(var Message : TMessage);  message CM_COLORIT;
    procedure WMNCHITTEST(var Message : TMessage);message WM_NCHITTEST;
    procedure WMKEYUP(var Message : TMessage);    message WM_KEYUP;
  protected
    procedure SetColorProperties(Value: TIAColorRichItems); virtual;
    procedure SetColorDelay(Value: Word);                   virtual;
    procedure SetColorRange(Value: Integer);                virtual;
    procedure SetColorRangePrimary(Value: Integer);         virtual;
    procedure Loaded;                                       override;
    procedure SetOnChange(Value : TNotifyEvent);            virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy;                    override;
    procedure CreateHandle;                 override;
    procedure RefreshColors(Itrycolorlater : boolean = true);
    procedure RefreshColorsAsFirstLoad(Newdata : string);
  published
    property ColorProperties   : TIAColorRichItems read FColorProperties   write SetColorProperties;
    property ColorDelay        : Word              read FColorDelay        write SetColorDelay        default 1500;
    property ColorRange        : Integer           read FColorRange        write SetColorRange        default 100;
    property ColorRangePrimary : Integer           read FColorRangePrimary write SetColorRangePrimary default 500;
    property OnChange          : TNotifyEvent      read FonChange          write SetonChange;
  end;

procedure Register;

implementation

{ TIAcolorRichEdit }

procedure TIAcolorRichEdit.AddDifference(Adiff: TIAdiapasonColor);
var TempDiff : array of TIAdiapasonColor;
    i : integer;
begin
  setlength(TempDiff,length(FDifference));
  for i:=0 to length(FDifference)-1 do
    begin
      TempDiff[i].Dbegin:=FDifference[i].Dbegin;
      TempDiff[i].Dend:=FDifference[i].Dend;
      TempDiff[i].Dindex:=FDifference[i].Dindex;
    end;
  setlength(FDifference,length(FDifference)+1);
  for i:=0 to length(FDifference)-2 do
    begin
      FDifference[i].Dbegin:=TempDiff[i].Dbegin;
      FDifference[i].Dend:=TempDiff[i].Dend;
      FDifference[i].Dindex:=TempDiff[i].Dindex;
    end;
  FDifference[length(FDifference)-1].Dbegin:=Adiff.Dbegin;
  FDifference[length(FDifference)-1].Dend:=Adiff.Dend;
  FDifference[length(FDifference)-1].Dindex:=Adiff.Dindex;
end;

procedure TIAcolorRichEdit.CMCOLORIT(var Message: TMessage);
begin
  inherited;
  FColorTimer.Suspend;
  TryColorItems;
end;

procedure TIAcolorRichEdit.ColorAndSelect(Adiff: TIAdiapasonColor);
begin
  if Adiff.Dbegin<length(text) then
    begin
      selstart:=Adiff.Dbegin;
      sellength:=Adiff.Dend-Adiff.Dbegin;
      if Adiff.Dindex<0 then
        begin
          SelAttributes.Size:=font.Size;
          SelAttributes.Color:=font.Color;
          SelAttributes.Style:=font.Style;
          SelAttributes.Charset:=font.Charset;
          SelAttributes.name:=font.name;
        end else
        begin
          SelAttributes.Color:=ColorProperties[Adiff.Dindex].FontColor;
          SelAttributes.Size :=ColorProperties[Adiff.Dindex].FontSize;
          SelAttributes.Name :=ColorProperties[Adiff.Dindex].FontName;
          SelAttributes.Style:=ColorProperties[Adiff.Dindex].FontStyle;
        end;
      sellength:=0;  
    end;
end;


procedure TIAcolorRichEdit.ColorNumber(Index: integer);
begin
  case ColorProperties[Index].ItemClass of
    iaicWord : ColorNumberWord(Index);
    iaicRange : ColorNumberRange(Index);
    iaicBrackets : ColorNumberBrackets(Index);
  end;
end;

procedure TIAcolorRichEdit.ColorNumberBrackets(Index: integer);
var ITryButItWouldNot : Boolean;//Copyright by Disney (Lion King 2)... :-)
    WeBeginAt,WeSeachAtLength,LeftBPosition,Lpos,Rpos,weFindLeft,WefindRight,Weseachat,Lsearch : integer;
    wemustgo : boolean;
    ASelB,ASelL,Weendat : integer;
begin
  if Currdelta>0 then
    begin
      Webeginat:=Currpos-30-Currdelta;
      Weendat:=Currpos+30+Currdelta+CurrL;
    end else
    begin
      webeginat:=0;
      weendat:=length(Fstring)-1;
   end;
  if WebeginAt<0 then WeBeginAt:=0;
  if Weendat>=length(fstring) then WeEndAt:=length(fstring)-1;

  ITryButItWouldNot:=false;
  if (ColorProperties[Index].LeftBracket<>'') and (ColorProperties[Index].RightBracket='') then
      ColorProperties[Index].RightBracket:=chr(13);
  if (ColorProperties[Index].LeftBracket<>'') and (ColorProperties[Index].RightBracket<>'') then
    begin
      WeSeachAtLength:=weendat-WeBeginAt;
      repeat
        if ColorProperties[Index].MatchCase=iamcYes then
          begin
            LeftBPosition:=FindText1(WeBeginAt,WeSeachAtLength,[stMatchCase],ColorProperties[Index].LeftBracket);
          end else
          begin
            LeftBPosition:=FindText1(WeBeginAt,WeSeachAtLength,[],ColorProperties[Index].LeftBracket);
          end;
        if LeftBPosition<0 then
          begin
            ITryButItWouldNot:=True;
          end else
          begin
            Rpos:=0;Lpos:=0;WeSeachAt:=LeftBposition+length(ColorProperties[Index].LeftBracket);
            Lsearch:=length(Fstring)-WeSeachAt+2*lines.Count;wemustgo:=false;
            repeat
              if ColorProperties[Index].MatchCase=iamcYes then
                begin
                  weFindLeft :=FindText1(WeSeachAt,Lsearch,[stMatchCase],ColorProperties[Index].LeftBracket);
                  weFindRight:=FindText1(WeSeachAt,Lsearch,[stMatchCase],ColorProperties[Index].RightBracket);
                end else
                begin
                  weFindLeft :=FindText1(WeSeachAt,Lsearch,[],ColorProperties[Index].LeftBracket);
                  weFindRight:=FindText1(WeSeachAt,Lsearch,[],ColorProperties[Index].RightBracket);
                end;

              if weFindLeft<0 then
                begin
                  if wefindright<0 then
                    begin
                      wemustgo:=true;
                    end else
                    begin
                      inc(Rpos);
                      if (Rpos-Lpos)=1 then wemustgo:=true;
                      WeSeachAt:=wefindRight+length(ColorProperties[Index].RightBracket);
                      Lsearch:=length(Fstring)-WeSeachAt+2*lines.Count;
                    end;
                end else
                begin
                  if wefindright<0 then
                    begin
                      inc(LPos);
                      WeSeachAt:=wefindLeft+length(ColorProperties[Index].LeftBracket);
                      Lsearch:=length(Fstring)-WeSeachAt+2*lines.Count;
                    end else
                    begin
                      if wefindright<weFindLeft then
                        begin
                          inc(Rpos);
                          if (Rpos-Lpos)=1 then wemustgo:=true;
                          WeSeachAt:=wefindRight+length(ColorProperties[Index].RightBracket);
                          Lsearch:=length(Fstring)-WeSeachAt+2*lines.Count;
                        end else
                        begin
                          inc(LPos);
                          WeSeachAt:=wefindLeft+length(ColorProperties[Index].LeftBracket);
                          Lsearch:=length(Fstring)-WeSeachAt+2*lines.Count;
                          if wefindright=weFindLeft then
                            begin
                              Rpos:=Lpos+1;wemustgo:=true;
                            end;
                        end;
                    end;
                end;

            until WEmustgo;
            if (Rpos-Lpos)=1 then
              begin
                ASelB:=LeftBPosition+length(ColorProperties[Index].LeftBracket);
                ASelL:=wefindRight-ASelB;
                if (currdelta<=0) or (((ASelB+AselL)>=(currpos-currdelta)) and (ASelB<=(currpos+currdelta+CurrL)))then
                RedefineColor(ASelB,ASelL,Index);
                WeBeginAt:=wefindRight+length(ColorProperties[Index].RightBracket);
                WeSeachAtLength:=weendat-WeBeginAt;
              end else
              begin
                ASelB:=LeftBPosition+length(ColorProperties[Index].LeftBracket);
                ASelL:=length(Fstring)-ASelB;
                if (currdelta<=0) or (((ASelB+AselL)>=(currpos-currdelta)) and (ASelB<=(currpos+currdelta+CurrL)))then
                RedefineColor(ASelB,ASelL,Index);
                ITryButItWouldNot:=TRUE;
              end;
          end;

      until ITryButItWouldNot;
    end;
end;

procedure TIAcolorRichEdit.ColorNumberRange(Index: integer);
var ITryButItWouldNot : Boolean;//Copyright by Disney (Lion King 2)... :-)
    WeSeachAtLength : integer;
    WeFindAt,TempFind,dindex,WeBeginAt,weendat : integer;
    j : byte;
    ASelB,ASelL : integer;
    ColoredArray : set of CHAR;
begin
  if (Index>0) and (ColorProperties[Index-1].ItemClass=iaicRange) and (ColorProperties[Index-1].WordName='next') then
    begin
      //Do Nothing....
    end else
    begin
      dindex:=0;ColoredArray:=[];
      while (cardinal(Index+dindex+1)<=(self.ColorProperties.ItemsCount)) and (ColorProperties[Index+dindex].ItemClass=iaicRange) do
        begin
          for j:=0 to 255 do
            begin
              if (j>=ColorProperties[Index+dindex].RangeBegin) and  (j<=ColorProperties[Index+dindex].RangeEnd) then
                ColoredArray:=ColoredArray+[CHR(j)];
            end;
          if ColorProperties[Index+dindex].WordName='next' then
            begin
              inc(dindex);
            end else
            begin
              break;
            end;
        end;

       if Currdelta>0 then
         begin
           Webeginat:=Currpos-30-Currdelta;
           Weendat:=Currpos+30+Currdelta+CurrL;
         end else
         begin
           webeginat:=0;
           weendat:=length(Fstring)-1;
         end;



      if WebeginAt<0 then WeBeginAt:=0;
      if Weendat>=length(fstring) then WeEndAt:=length(fstring)-1;
      ITryButItWouldNot:=false;

      WeSeachAtLength:=WeEndAt-WeBeginAt+1;;
      repeat
        if ColorProperties[Index].LeftBracket='' then
        begin
          WefindAt:=-1;
          for j:=0 to 255 do
            begin
              if (CHR(j)) in ColoredArray then
                begin
                  TempFind:=FindText1(WeBeginAt,WeSeachAtLength,[],chr(j));
                  if ((TempFind>=0) and ((WeFindAt=-1) or(TempFind<WefindAt))) then
                    begin
                      WefindAt:=TempFind;
                    end;
                end;
            end;
        end else
        begin
          if ColorProperties[Index].MatchCase=iamcYes then
            begin
              WeFindAt:=FindText1(WeBeginAt,WeSeachAtLength,[stMatchCase],ColorProperties[Index].LeftBracket);
            end else
            begin
              WeFindAt:=FindText1(WeBeginAt,WeSeachAtLength,[],ColorProperties[Index].LeftBracket);
            end;
        end;
        if WeFindAt<0 then
          begin
            ITryButItWouldNot:=TRUE;//Kiarra...
          end else
          begin
            // we try to define word length....
            dindex:=length(ColorProperties[Index].LeftBracket);
            while (length(Fstring)>=(WEFindAt+dindex+1)) and (Fstring[WEFindAt+dindex+1] in coloredarray) do inc(dindex);

            if ColorProperties[Index].WholeWord=iawwYes then
              begin
                if ((WEFindAt=0) or (Fstring[WEFindAt] in TIASpaseSet)) and
                   (((wefindAT+dindex)>=length(Fstring)) or (Fstring[wefindAT+dindex+1] in TIASpaseSet)) then
                  begin
                    //we draw items...
                    if dindex=0 then
                      begin
                        WeBeginAt:=wefindAT+1;
                        WeSeachAtLength:=WeEndAt-WeBeginAt+1;;
                      end else
                      begin
                        ASelB:=wefindAT+length(ColorProperties[Index].LeftBracket);
                        ASelL:=dindex-length(ColorProperties[Index].LeftBracket);
                        if (currdelta<=0) or ((ASelB>=(currpos-currdelta)) and ((ASelB+AselL)<=(currpos+currdelta+CurrL)))then
                        RedefineColor(ASelB,ASelL,Index);
                        WeBeginAt:=wefindAT+dindex;
                        WeSeachAtLength:=WeEndAt-WeBeginAt+1;;
                      end;
                  end else
                  begin
                    //we do not draw items
                    if dindex=0 then
                      begin
                        WeBeginAt:=wefindAT+1;
                        WeSeachAtLength:=WeEndAt-WeBeginAt+1;;
                      end else
                      begin
                        WeBeginAt:=wefindAT+dindex;
                        WeSeachAtLength:=WeEndAt-WeBeginAt+1;;
                      end;
                  end;
              end else
              begin
                //we draw items...
                if dindex=0 then
                  begin
                    WeBeginAt:=wefindAT+1;
                    WeSeachAtLength:=WeEndAt-WeBeginAt+1;;
                  end else
                  begin
                    ASelB:=wefindAT+length(ColorProperties[Index].LeftBracket);
                    ASelL:=dindex-length(ColorProperties[Index].LeftBracket);
                    if (currdelta<=0) or ((ASelB>=(currpos-currdelta)) and ((ASelB+AselL)<=(currpos+currdelta+CurrL)))then
                    RedefineColor(ASelB,ASelL,Index);
                    WeBeginAt:=wefindAT+dindex;
                    WeSeachAtLength:=WeEndAt-WeBeginAt+1;;
                  end;
              end;
          end;

      until ITryButItWouldNot;
    end;

end;

procedure TIAcolorRichEdit.ColorNumberWord(Index: integer);
var ITryButItWouldNot : Boolean;//Copyright by Disney (Lion King 2)... :-)
    WeSeachAtLength,WeBeginAt,WeEndAt,WeFindAt,WeFindLeftBracketAt,Brbegin,Lbegin : integer;
    Options : TSearchTypes;
begin
 if Currdelta>0 then
   begin
     Webeginat:=Currpos-30-Currdelta;
     Weendat:=Currpos+30+Currdelta+CurrL;
   end else
   begin
     webeginat:=0;
     weendat:=length(Fstring)-1;
   end;
 if WebeginAt<0 then WeBeginAt:=0;
 if Weendat>=length(fstring) then WeEndAt:=length(fstring)-1;
 options:=[];
 if ColorProperties[Index].WholeWord=iawwYes then options:=options+[stWholeWord];
 if ColorProperties[Index].MatchCase=iamcYes then options:=options+[stMatchCase];
 WeSeachAtLength:=WeEndAt-WeBeginAt+1;ITryButItWouldNot:=false;
  repeat
    WeFindAt:=FindText1(WeBeginAt,WeSeachAtLength,options,ColorProperties[Index].WordName);
    if WeFindAt<0 then
      begin
        ITryButItWouldNot:=TRUE;
      end else
      begin
        // if we have the brackets we need to check it....
        if ColorProperties[Index].LeftBracket='' then
          begin
            if (currdelta<=0) or ((WeFindAt>=(currpos-currdelta)) and
            ((WeFindAt+length(ColorProperties[Index].WordName))<=(currpos+currdelta+CurrL)))then
            RedefineColor(WeFindAt,length(ColorProperties[Index].WordName),Index);
            WeBeginAt:=WeFindAt+length(ColorProperties[Index].WordName);
            WeSeachAtLength:=WeEndAt-WeBeginAt+1;
          end else
          begin
            if ColorProperties[Index].RightBracket='' then
              begin
                WeFindLeftBracketAt:=FindText1(0,length(Fstring),[],ColorProperties[Index].LeftBracket);
                if (WeFindLeftBracketAt>=0) and (WeFindLeftBracketAt<WeFindAt) then
                  begin
                    if (currdelta<=0) or ((WeFindAt>=(currpos-currdelta)) and
                   ((WeFindAt+length(ColorProperties[Index].WordName))<=(currpos+currdelta+CurrL)))then
                    RedefineColor(WeFindAt,length(ColorProperties[Index].WordName),Index);
                    WeBeginAt:=WeFindAt+length(ColorProperties[Index].WordName);
                    WeSeachAtLength:=WeEndAt-WeBeginAt+1;
                  end;
              end else
              begin
                BrBegin:=0;Lbegin:=0;
                repeat
                  BrBegin:=FindText1(BrBegin,WeFindAt-BrBegin,[],ColorProperties[Index].RightBracket);
                  if (BrBegin>=0) and (BrBegin<WeFindAt) then
                    begin
                      Lbegin:=Brbegin;
                      inc(BrBegin,length(ColorProperties[Index].RightBracket));
                    end;
                until (BrBegin<0) or (BrBegin>WeFindAt);
                BrBegin:=FindText1(Lbegin,WeFindAt-Lbegin,[],ColorProperties[Index].LeftBracket);
                if Brbegin>=0 then
                  begin
                    if (currdelta<=0) or ((WeFindAt>=(currpos-currdelta)) and
                   ((WeFindAt+length(ColorProperties[Index].WordName))<=(currpos+currdelta+CurrL)))then
                    RedefineColor(WeFindAt,length(ColorProperties[Index].WordName),Index);
                    WeBeginAt:=WeFindAt+length(ColorProperties[Index].WordName);
                    WeSeachAtLength:=WeEndAt-WeBeginAt+1;
                  end else
                  begin
                    if abs(currpos-Wefindat)<currdelta then
                    RedefineColor(WeFindAt,length(ColorProperties[Index].WordName),-1);
                    WeBeginAt:=WeFindAt+length(ColorProperties[Index].WordName);
                    WeSeachAtLength:=WeEndAt-WeBeginAt+1;
                  end;
              end;
          end;
      end;
  until ITryButItWouldNot;
end;

constructor TIAcolorRichEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FColorDelay:=1500;
  FColorRange:=100;
  FcolorRangePrimary:=500;
  FUnderColor:=false;
  setlength(FSc,0);
  FMaxUndos:=16;
  Currdelta:=-1;
  Self.ControlStyle:=Self.ControlStyle+[csOpaque];
  FColorProperties:=TIAColorRichItems.Create;
  FColorProperties.onChange:=FontGhanged;
  self.Font.OnChange:=FontGhanged;
  FColorTimer:=TIAColortimer.Create(TRUE);
  FColorTimer.fmydelay:=1500;
end;

procedure TIAcolorRichEdit.CreateHandle;
begin
  inherited CreateHandle;
  FColorTimer.FHandle:=self.handle;
  inherited OnChange:=InternalGhanged;
end;

destructor TIAcolorRichEdit.Destroy;
begin
  if FColorTimer.Suspended then
    begin
      FColorTimer.Terminate;
      FColorTimer.Resume;
      FColorTimer.WaitFor;
    end else
    begin
      FColorTimer.Terminate;
      FColorTimer.WaitFor;
    end;
  FColorProperties.Free;
  inherited Destroy;
end;

function TIAcolorRichEdit.FindText1(StartPos,Length1: Integer; Options: TSearchTypes;ss: string): Integer;
var s : string;
    icango : boolean;
    Ifindat,SearchLength : integer;
begin
  // we find the Fstring except any cursor movies....
  if not (stMatchCase in Options) then
    begin
      s:=copy(FstringU,StartPos+1,Length1);
      CharUpperBuff(@(ss[1]),length(ss));
    end else
    begin
      s:=copy(Fstring,StartPos+1,Length1);
    end;
  ifindat:=AnsiPos(ss,s);
  result:=ifindat-1;
  if result>=0 then inc(result,StartPos);
  if stWholeWord in Options then
    begin
      icango:=false;SearchLength:=System.length(ss);
      repeat
        if ifindat=0 then break;
        if result>=0 then
          begin
            if ((result>0) and (([Fstring[result]]+TIASpaseSet)<>TIASpaseSet)) or
               (((result+SearchLength)<(System.length(Fstring)-1)) and
                (([Fstring[result+SearchLength+1]] + TIASpaseSet)<>TIASpaseSet)) then  result:=-1;
          end;
        if result<0 then
          begin
            inc(startpos,ifindat+SearchLength-1);
            s:=copy(s,ifindat+SearchLength,Length(s)-ifindat-SearchLength+1);
            if s='' then
              begin
                icango:=true;
              end else
              begin
                ifindat:=AnsiPos(ss,s);
                result:=ifindat-1;
                if result>=0 then inc(result,StartPos);
              end;
          end else
          begin
            icango:=true;
          end;
      until icango;
    end;
end;

procedure TIAcolorRichEdit.FontGhanged(Sender: TObject);
begin
  selStart :=0;
  SelLength:=length(Text);
  SelAttributes.Size:=font.Size;
  SelAttributes.Color:=font.Color;
  SelAttributes.Style:=font.Style;
  SelAttributes.Charset:=font.Charset;
  SelAttributes.name:=font.name;
  sellength:=0;
  setlength(fsc,0);
  Currdelta:=-1;
  if FColorTimer.Suspended then FColorTimer.Resume;
end;

procedure TIAcolorRichEdit.InternalGhanged(Sender: TObject);
begin
  //Here we colour the content....
  if FString<>text then
    begin
      if (abs(length(FString)-length(text))>(FColorRange)) or (FColorRange<=0) then
        begin
          if FcolorRangePrimary>0 then Currdelta:=FcolorRangePrimary*2 else Currdelta:=-1;
          FLastPosition:=SelStart;
          TryColorItems;
        end else
        begin
          if FColorTimer.Suspended then
            begin
              Currdelta:=FColorRange;
              FColorTimer.Resume;
            end;  
    end;
    end;

  if Assigned(FOnChange) then FOnChange(self);
end;

procedure TIAcolorRichEdit.Loaded;
begin
  inherited Loaded;
  RefreshColorsAsFirstLoad(Text);
end;

procedure TIAcolorRichEdit.RedefineColor(Index, L, Connect: integer);
var i : integer ;
begin
  for i:=Index to Index+L-1    do if i<length(Fs) then Fs[i]:=Connect;
end;

procedure TIAcolorRichEdit.RefreshColors(Itrycolorlater : boolean = true);
begin
  selStart :=0;
  SelLength:=length(Text);
  SelAttributes.Size:=font.Size;
  SelAttributes.Color:=font.Color;
  SelAttributes.Style:=font.Style;
  SelAttributes.Charset:=font.Charset;
  SelAttributes.name:=font.name;
  sellength:=0;
  setlength(fsc,0);
  if not Itrycolorlater then begin Currdelta:=-1; TryColorItems; end;
end;

procedure TIAcolorRichEdit.RefreshColorsAsFirstLoad(Newdata : string);
begin
  selStart :=0;
  SelLength:=length(Text);
  SelAttributes.Size:=font.Size;
  SelAttributes.Color:=font.Color;
  SelAttributes.Style:=font.Style;
  SelAttributes.Charset:=font.Charset;
  SelAttributes.name:=font.name;
  sellength:=0;
  setlength(fsc,0);
  text:=Newdata;
//  if FcolorRangePrimary>0 then Currdelta:=FcolorRangePrimary*2 else Currdelta:=-1;
//  FLastPosition:=SelStart;
//  TryColorItems;
end;

procedure TIAcolorRichEdit.SetColorDelay(Value: Word);
begin
  if fColorDelay<>Value  then
    begin
      FColorDelay := Value;
      FColorTimer.Suspend;
      FColorTimer.fmydelay:=Value;
      FColorTimer.Resume;
    end;
end;

procedure TIAcolorRichEdit.SetColorProperties(Value: TIAColorRichItems);
begin
  if Value<>nil then
    begin
      FColorProperties.Assign(Value);
    end;
end;

procedure Register;
begin
  RegisterComponents('TIA', [TIAcolorRichEdit]);
end;

procedure TIAcolorRichEdit.SetColorRange(Value: Integer);
begin
  FColorRange := Value;
end;

procedure TIAcolorRichEdit.SetColorRangePrimary(Value: Integer);
begin
  FColorRangePrimary := Value;
end;

procedure TIAcolorRichEdit.SetOnChange(Value: TNotifyEvent);
begin
  FOnChange:=Value;
end;

procedure TIAcolorRichEdit.TryColorItems(DoAtPos : Boolean = false);
var NnoShow,Nshowed : integer;
    i,j : integer;
    showed : array of Boolean;
    currpower : integer;
    first : boolean;
    MyDiff : TIAdiapasonColor;
    i1,i2,i1x : integer;
begin
  if FUnderColor then
    begin

    end else
    begin
      FUnderColor:=True;
      if (FColorProperties.ItemsCount>0) and (text<>'') then
        begin
          setlength(showed,FColorProperties.ItemsCount);
          for i:=0 to FColorProperties.ItemsCount-1 do showed[i]:=false;
          NnoShow:=0;
          for i:=0 to FColorProperties.ItemsCount-1 do if FColorProperties[i].Priority<0 then begin showed[i]:=TRUE; inc(NnoShow); end;
          if cardinal(NnoShow)<FColorProperties.ItemsCount then
            begin
              CurrPos:=SelStart;CurrL:=SelLength;
              setlength(Fs,0);
              setlength(Fs,length(text));FString:=Text;FStringU:=CharUpper(@(FString[1]));
              for i:=0 to length(Fs)-1 do Fs[i]:=-1;
              if CurrDelta>0 then
                begin
                  if length(Fs)>=length(Fsc) then
                    begin
                      for i:=0 to length(fsc)-1 do
                        begin

                          if i<=currpos then
                            begin
                              if ((i<=(Currpos-Currdelta)) or (i>=(Currpos+Currdelta+CurrL))) and (Currdelta>0) then
                              fs[i]:=fsc[i];
                            end else
                            begin
                              if (((i+(length(fs)-length(fsc)))<=(Currpos-Currdelta))
                                or ((i+(length(fs)-length(fsc)))>=(Currpos+Currdelta+CurrL)))
                                and (Currdelta>0) then
                              fs[i+(length(fs)-length(fsc))]:=fsc[i];
                            end;
                        end;
                    end else
                    begin
                      for i:=0 to length(fs)-1 do
                        begin
                          if ((i<=(Currpos-Currdelta)) or (i>=(Currpos+Currdelta+CurrL))) and (Currdelta>0) then
                          if i<=currpos then
                            begin
                              fs[i]:=fsc[i];
                            end else
                            begin
                              fs[i]:=fsc[(length(fsc)-length(fs))+i];
                            end;
                        end;
                    end;
                end;

              for i:=1 to integer(FColorProperties.ItemsCount)-NnoShow do
                begin
                  first:=true;NShowed:=0;currpower:=-1;
                  for j:=FColorProperties.ItemsCount-1 downto 0 do
                    begin
                      if not showed[j] then
                        begin
                          if first then
                            begin
                              Nshowed:=j;currpower:=FColorProperties[j].Priority;
                              first:=false;
                            end else
                            begin
                              if currpower>=FColorProperties[j].Priority then
                                begin
                                  Nshowed:=j;currpower:=FColorProperties[j].Priority;
                                end;
                            end;
                        end;
                    end;
                    // we define the current priority redefiner...
                    showed[Nshowed]:=True;
                    ColorNumber(NShowed);
                end;
              //we get new colors disposition...
              //we must to fill the differences...
              setlength(FDifference,0);

              if length(Fs)>length(FSc) then
                begin

                  selStart:=currpos+currL-(length(Fs)-length(FSc));
                  SelLength:=length(Fs)-length(FSc);
                  SelAttributes.Size:=font.Size;
                  SelAttributes.Color:=font.Color;
                  SelAttributes.Style:=font.Style;
                  SelAttributes.Charset:=font.Charset;
                  SelAttributes.name:=font.name;
                  SelLength:=0;
                  SelStart:=currpos;
                  i1:=0;
                  while (i1+1)<=length(Fs) do
                    begin
                      if i1<(currpos-1-(length(Fs)-length(FSc))+currl) then
                        begin
                          if i1<length(FSc) then
                            begin
                              I1x:=Fsc[i1];
                            end else
                            begin
                              I1x:=-1;
                            end;
                        end else
                        begin
                          if (i1-currpos)>=(length(Fs)-length(FSc)) then
                            begin
                              if (i1-(length(Fs)-length(FSc)))<length(FSc) then
                                begin
                                  I1x:=Fsc[i1-(length(Fs)-length(FSc))];
                                end else
                                begin
                                  I1x:=-1;
                                end;
                            end else
                            begin
                              I1x:=-1;
                            end;
                        end;

                      if Fs[i1]<>I1x then
                        begin
                          i2:=i1;
                          while ((i2<length(Fs)) and (FS[i1]=Fs[i2]) ) do inc(i2);
                          mydiff.Dbegin:=i1;
                          mydiff.Dend:=i2;
                          mydiff.Dindex:=FS[i1];
                          AddDifference(mydiff);
                          i1:=i2;
                        end else
                        begin
                          inc(i1);
                        end;
                    end;
                end else
                begin
                  if length(Fs)=length(Fsc) then
                    begin
                      i1:=0;
                      while (i1+1)<=length(Fs) do
                        begin
                          if Fs[i1]<>Fsc[i1] then
                            begin
                              i2:=i1;
                              while ((i2<length(Fs)) and (FS[i1]=Fs[i2])) do inc(i2);
                              mydiff.Dbegin:=i1;
                              mydiff.Dend:=i2;
                              mydiff.Dindex:=FS[i1];
                              AddDifference(mydiff);
                              i1:=i2;
                            end else
                            begin
                              inc(i1);
                            end;
                        end;
                    end else
                    begin
                      i1:=0;
                      while (i1+1)<=length(Fs) do
                        begin
                          if i1<currpos then I1x:=Fsc[i1] else I1x:=Fsc[i1-(length(Fs)-length(FSc))];
                          if Fs[i1]<>I1x then
                            begin
                              i2:=i1;
                              while ((i2<length(Fs)) and (FS[i1]=Fs[i2])) do inc(i2);
                              mydiff.Dbegin:=i1;
                              mydiff.Dend:=i2;
                              mydiff.Dindex:=FS[i1];
                              AddDifference(mydiff);
                              i1:=i2;
                            end else
                            begin
                              inc(i1);
                            end;
                        end;
                    end;
                end;
              // we define the differences
              // we must to color it..
              if length(FDifference)<>0 then
                begin
                  for i:=0 to length(FDifference)-1 do
                    begin
                      ColorAndSelect(FDifference[i]);
                    end;
                  selStart:=currpos;
                  sellength:=0;
                  SelAttributes.Size:=font.Size;
                  SelAttributes.Color:=font.Color;
                  SelAttributes.Style:=font.Style;
                  SelAttributes.Charset:=font.Charset;
                  SelAttributes.name:=font.name;
                  setlength(Fsc,0);
                  setlength(Fsc,length(Fs));
                  for i:=1 to length(Fs) do
                    begin
                      Fsc[i-1]:=Fs[i-1];
                    end;
                  setlength(Fs,0);  
                end else
                begin
                  // no differences...
                  setlength(Fsc,0); 
                  setlength(Fsc,length(Fs));
                  for i:=1 to length(Fs) do
                    begin
                      Fsc[i-1]:=Fs[i-1];
                    end;
                  setlength(Fs,0);
                end;
            end;
        end;
      FUnderColor:=false;
    end;
end;

procedure TIAcolorRichEdit.WMKEYUP(var Message: TMessage);
begin
  inherited;
  if (abs(selstart-FLastPosition)>FColorRangePrimary) and (FColorRangePrimary>0) then
    begin
      Currdelta:=FcolorRangePrimary;
      FLastPosition:=SelStart;
      TryColorItems;
    end;
  Message.Result:=0;  
end;

procedure TIAcolorRichEdit.WMNCHITTEST(var Message: TMessage);
begin
  inherited;
  if (abs(selstart-FLastPosition)>FColorRangePrimary) and (FColorRangePrimary>0) then
    begin
      Currdelta:=FcolorRangePrimary;
      FLastPosition:=SelStart;
      TryColorItems;
    end;
end;

end.
